@@ -625,11 +625,7 @@ class Product(ExprWithIntLimits):
                 return S.One
         else:
             x, *_ = limit
-        import sympy
         function = self.expr
-        if isinstance(function, sympy.exp):
-            function = function.as_Mul()
-
         independent, dependent = function.as_independent(x, as_Add=False)
         if independent == S.One:
             return self
@@ -1023,11 +1019,7 @@ class MatProduct(ExprWithIntLimits, MatrixExpr):
                 return Identity(self.shape[0])
         else:
             x, *_ = limit
-        import sympy
         function = self.expr
-        if isinstance(function, sympy.exp):
-            function = function.as_Mul()
-
         independent, dependent = function.as_independent(x, as_Add=False)
         if independent == S.One:
             return self
