@@ -155,7 +155,10 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
     def doit(self, **hints):
         if hints.get('deep', True):
-            f = self.expr.doit(**hints)
+            try:
+                f = self.expr.doit(**hints)
+            except:
+                f = self.expr
         else:
             f = self.expr
 
@@ -835,8 +838,17 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             
                     return self.func(function, (x, a._subs(old, new, **hints), b._subs(old, new, **hints))).simplify()
 
-                if (x.is_Sliced or x.is_Indexed) and x.base != old:
+                if x.is_Sliced and x.base != old:
+                    try:
+                        x = x._subs(old, new, **hints)
+                    except Exception as e:
+                        if e.args[0] == 'empty slices':
+                            return function
+                        raise e
+
+                elif x.is_Indexed and x.base != old:
                     x = x._subs(old, new, **hints)
+
                 domain = self.expr.domain_defined(x)
                 _domain = function.domain_defined(x)
                 if domain != _domain:
@@ -892,31 +904,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
             return self.func(function, (_x))
 
-        elif len(self.limits) == 0:
-            function = self.expr._subs(old, new, **hints)
-
-            return self.func(function, *self.limits)
-        else:
-            limits = [*self.limits]
-            hit = False
-            
-            for i, limit in enumerate(limits):
-                x, *ab = limit
-#                 p = old.as_poly(x)
-#                 if p is None:
-#                     continue
-#                 
-#                 if p.degree() == 0:
-                limits[i] = (x, *[a._subs(old, new, **hints) for a in ab])
-                hit |= limits[i] != self.limits[i] 
-            
-            function = self.expr._subs(old, new, **hints)
-            hit |= function != self.expr
-            
-            if hit:
-                return self.func(function, *limits)
-
-        return self
+        return AddWithLimits._subs(self, old, new, **hints)
 
     def try_sub(self, autre):
         if isinstance(autre, Sum) and self.expr == autre.expr and len(self.limits) == len(autre.limits) == 1 and len(self.limits[0]) == len(autre.limits[0]) == 3:
@@ -1507,7 +1495,7 @@ domain & limit[1] = %s
     latex_name_of_operator = 'sum'
     
     def _eval_is_finite(self):
-        function = self.expr                
+        function = self.expr
         for x, domain in self.limits_dict.items():
             if not isinstance(domain, list) and not domain.is_bool:
                 if domain.is_infinite:
