@@ -904,7 +904,30 @@ class Add(Expr, AssocOp):
 
     def _eval_transpose(self, axis=-1):
         if axis == self.default_axis:
-            return self.func(*[t.T for t in self.args])
+            scalar = []
+            vector = []
+            matrix = []
+            for arg in self.args:
+                if not arg.shape:
+                    scalar.append(arg)
+                elif len(arg.shape) == 1:
+                    vector.append(arg)
+                else:
+                    matrix.append(arg)
+                
+            if not vector:
+                return self.func(*(arg.T for arg in self.args))
+
+            from sympy import OneMatrix 
+            one = OneMatrix(*self.shape)
+            vector = [v * one for v in vector]
+            matrix += vector
+
+            matrix = self.func(*(arg.T for arg in matrix))
+            if scalar:
+                matrix += self.func(*scalar)
+
+            return matrix
 
     def __neg__(self):
         return self * (-1)
@@ -1539,7 +1562,7 @@ class Add(Expr, AssocOp):
             sign = ""
         return sign + ' '.join(l)
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         if self.is_upper or self.is_lower:
             from sympy.concrete.products import Product
             i = self.generate_var(integer=True)
@@ -1820,7 +1843,11 @@ class Add(Expr, AssocOp):
 
             return Add(*modified, *constants, unmodified)
         return self
-    
+
+    @property
+    def is_comparable(self):
+        return all(arg.is_comparable for arg in self.args)
+
     
 from .mul import Mul, _keep_coeff, prod
 from sympy.core.numbers import Rational
