@@ -312,7 +312,8 @@ class Tuple(Basic):
             return (x, (Range if x.is_integer else Interval)(*ab))
         return self
     
-    def coerce_setlimit(self, function=None):
+    @classmethod
+    def to_coerce_setlimit(cls, self, function=None):
         x, *ab = self
         if not ab:
             if function is None:
@@ -328,9 +329,21 @@ class Tuple(Basic):
             if b.is_set:
                 domain = b & x.domain_conditioned(a)
             else:
-                from sympy import Interval, Range
-                domain = (Range if x.is_integer else Interval)(a, b)
+                if x.is_integer:
+                    from sympy import Range
+                    domain = Range(a, b)
+                else:
+                    from sympy import Interval
+                    if a <= b:
+                        return x, Interval(a, b, right_open=True), 1
+                    elif a > b:
+                        return x, Interval(b, a, left_open=True), -1
+                    else:
+                        return
         return x, domain
+
+    def coerce_setlimit(self, function=None):
+        return Tuple.to_coerce_setlimit(self, function)
     
     def limit(self, x, xlim, dir=1):
         """ Compute limit x->xlim.
@@ -338,6 +351,19 @@ class Tuple(Basic):
         from sympy.series.limits import limit
         return Tuple(*[limit(f, x, xlim, dir) for f in self.args])
     
+    @classmethod
+    def is_nonemptyset(cls, limits):
+        for limit in limits:
+            args = Tuple.to_coerce_setlimit(limit)
+            if not args:
+                continue
+
+            x, domain, *dir = args
+            if not domain.is_nonempty:
+                return False
+
+        return True
+    
 
 converter[tuple] = lambda tup: Tuple(*tup)
 
