@@ -21,7 +21,7 @@ class Inference:
 
     @property    
     def T(self):
-        return Inference(self.cond.T, equivalent=self)        
+        return Inference(self.cond.T, equivalent=self)
 
     @property    
     def equivalent(self):
@@ -413,7 +413,7 @@ class Inference:
         if 'simplify' not in kwargs:
             kwargs['simplify'] = True
 
-        if all(eq.is_Equal for eq in args):
+        if all(eq.is_Equal or eq.is_Equivalent for eq in args):
             for eq in args:
                 old, new = eq.args
                  
@@ -481,7 +481,7 @@ class Inference:
         domain = old.domain_bounded
         if domain is not None and new not in domain:
             from sympy import NotElement
-            if self.is_ForAll:                
+            if self.is_ForAll:
                 assert old not in self.variables, 'not supported in built-in axioms, please employ proved theorems: algebra.all.imply.ou.subs'
                 if self.expr._has(old):
                     function = self.expr._subs(old, new) | NotElement(new, domain)
@@ -495,7 +495,7 @@ class Inference:
                     
                     cond = self.func(self.expr, *limits) | NotElement(new, domain)
             else:
-                _old = old.unbounded                
+                _old = old.unbounded
                 cond = self._subs(old, _old) | NotElement(_old, old.domain)
                 if _old != new:
                     cond = cond._subs(_old, new, **kwargs)
@@ -670,8 +670,8 @@ class Inference:
 
                 if other.is_nonzero:
                     return self.func(lhs, rhs, equivalent=self)
-                from sympy import Or
-                return Or(self.func(other, 0), self.func(lhs, rhs), equivalent=self)
+                from sympy import Or, ZeroMatrix
+                return Or(self.func(other, ZeroMatrix(*other.shape)), self.func(lhs, rhs), equivalent=self)
                             
             if other.is_positive:
                 return Inference(self.func(self.lhs / other, self.rhs / other), equivalent=self)
@@ -688,8 +688,8 @@ class Inference:
 
             if other.is_nonzero:
                 return self.func(lhs, rhs, equivalent=self)
-            from sympy import Or
-            return Or(self.func(other, 0), self.func(lhs, rhs), equivalent=self)
+            from sympy import Or, ZeroMatrix
+            return Or(self.func(other, ZeroMatrix(*other.shape)), self.func(lhs, rhs), equivalent=self)
         
     def __iter__(self):
         raise TypeError
@@ -774,6 +774,15 @@ class Inference:
             if equivalent is self:
                 if given.imply is not None:
                     return self.given_by(given.imply)
+
+                equivalent = equivalent_ancestor(given)
+                if len(equivalent) != 1:
+                    return False
+                [equivalent] = equivalent
+                
+                if self == equivalent:
+                    return True
+
                 return False
             self = equivalent
             
