@@ -8,7 +8,7 @@ from sympy.core.sympify import _sympify as _sympify_, sympify
 from sympy.core.basic import Basic
 from sympy.core.cache import cacheit
 from sympy.core.compatibility import ordered, default_sort_key
-from sympy.core.logic import fuzzy_and, _fuzzy_group
+from sympy.core.logic import _fuzzy_group
 from sympy.core.parameters import global_parameters
 from sympy.utilities.iterables import sift
 from sympy.multipledispatch.dispatcher import (Dispatcher,
@@ -526,6 +526,22 @@ class AssocOp(Basic):
 
         return self.func(*args)
 
+    @property
+    def variables(self):
+        s = set()
+        for a in self.args:
+            s |= {*a.variables}
+            
+        return s
+    
+    def yield_random_symbols(self):    
+        for arg in self.args:
+            yield from arg.yield_random_symbols()
+        
+    def domain_definition(self):
+        from sympy import And
+        return And(*(arg.domain_definition() for arg in self.args))
+
 
 class ShortCircuit(Exception):
     pass
@@ -674,13 +690,14 @@ class LatticeOp(AssocOp):
         res = AssocOp.of(self, cls)
         if res is None:
             if cls.is_LatticeOp:
-                a, b = cls.args
                 from sympy.core.of import Basic
-                cls = Basic.__new__(cls.func, b, a)
-                res = AssocOp.of(self, cls)
-                if isinstance(res, tuple):
-                    b, a = res
-                    return (a, b)
+                if isinstance(cls, Basic) or cls.is_IndexedOperator:
+                    a, b = cls.args
+                    cls = Basic.__new__(cls.func, b, a)
+                    res = AssocOp.of(self, cls)
+                    if isinstance(res, tuple):
+                        b, a = res
+                        return a, b
         return res 
 
 
