@@ -947,16 +947,15 @@ class Pow(Expr):
 
     def _eval_transpose(self, axis=-1):
         if axis == self.default_axis:
-            from sympy.functions.elementary.complexes import transpose
             i, p = self.exp.is_integer, self.base.is_complex
             if p:
-                return self.base ** self.exp
+                return self.base.T ** self.exp
             if i:
-                return transpose(self.base) ** self.exp
+                return self.base.T ** self.exp
             if i is False and p is False:
                 expanded = expand_complex(self)
                 if expanded != self:
-                    return transpose(expanded)
+                    return expanded.T
 
     def _eval_expand_power_exp(self, **hints):
         """a**(n + m) -> a**n*a**m"""
@@ -1292,9 +1291,30 @@ class Pow(Expr):
 
     def _eval_derivative(self, s):
         from sympy import log
-        dbase = self.base.diff(s)
-        dexp = self.exp.diff(s)
-        return self * (dexp * log(self.base) + dbase * self.exp / self.base)
+        base, exp = self.args
+        if s.shape:
+            if exp.shape:
+                dbase = base.diff(s)
+                dexp = exp.diff(s)
+                return self * (dexp * log(base) + dbase * exp / base)
+            elif base.shape:
+                dbase = base.diff(s)
+                dexp = exp.diff(s)
+                if len(self.shape) < len(dbase.shape):
+                    from sympy import Transpose
+                    self = Transpose.expand_dims(self, dbase.shape, len(s.shape))
+                    base = Transpose.expand_dims(base, dbase.shape, len(s.shape))
+
+                return self * (dexp * log(base) + dbase * exp / base)
+            else:
+                dbase = base.diff(s)
+                dexp = exp.diff(s)
+                return self * (dexp * log(base) + dbase * exp / base)
+                
+        else:
+            dbase = base.diff(s)
+            dexp = exp.diff(s)
+            return self * (dexp * log(base) + dbase * exp / base)
 
     def _eval_evalf(self, prec):
         base, exp = self.as_base_exp()
@@ -2023,7 +2043,28 @@ class Pow(Expr):
         b, e = self.args
         return b.torch ** e.torch
     
+    @staticmethod
+    def simplify_Lamda(self, squeeze=False):
+        expr, *limits = self.args
+        b, e = expr.args
+        if e.shape:
+            if not b.shape:
+                if e.has(*self.variables):
+                    lamda = self.func(e, *self.limits)
+                    _lamda = lamda.simplify()
+                    if lamda != _lamda:
+                        return expr.func(b, _lamda, evaluate=False)
+                
+        else:
+            if b.has(*self.variables):
+                lamda = self.func(b, *self.limits)
+                _lamda = lamda.simplify()
+                if lamda != _lamda:
+                    return expr.func(_lamda, e, evaluate=False)
+
+        return self
     
+
 from .add import Add
 from .numbers import Integer
 from .mul import Mul, _keep_coeff
