@@ -628,6 +628,21 @@ class Symbol(AtomicExpr, NotIterable):
         from sympy.stats.rv import pspace
         return pspace(self).symbol
 
+    @property
+    def surrogate(self):
+        assert self.is_random
+        from sympy.stats.symbolic_probability import Surrogate
+        return Surrogate(self)
+
+    def __and__(self, other):
+        """Overloading for & operator"""
+        if self.is_random and other.is_random:
+            if not other.is_bool:
+                other = other.as_boolean()
+            return self.as_boolean() & other
+
+        return super(Symbol, self).__and__(other)
+
     @cacheit
     def defun(self):
         if 'definition' in self._assumptions:
@@ -716,6 +731,10 @@ class Symbol(AtomicExpr, NotIterable):
         if Expr._has(self, pattern):
             return True
 
+        if pattern.is_Sliced:
+            if pattern.base == self:
+                return True
+
         if not (isinstance(pattern, ManagedProperties) or pattern.is_FunctionClass):
             if 'definition' in self._assumptions:
                 definition = self._assumptions['definition']
@@ -727,9 +746,6 @@ class Symbol(AtomicExpr, NotIterable):
                     return False
                 domain = self._assumptions['domain']
                 return domain._has(pattern)
-        if pattern.is_Sliced:
-            if pattern.base == self:
-                return True
         return False
 
     @property
@@ -909,7 +925,7 @@ class Symbol(AtomicExpr, NotIterable):
             if definition is not None and definition.is_Lamda:
                 if len(definition.limits) == 1 and definition.expr.is_RandomSymbol:
                     return definition[indices]
-                    
+
             boolean = indices < self.shape[0]
             assert boolean is True or boolean != False
             return Indexed(self, indices, **kw_args)
@@ -987,7 +1003,7 @@ class Symbol(AtomicExpr, NotIterable):
     def _sympystr(self, _): 
         return Symbol.sympystr(self.name)     
 
-    def _latex(self, p):
+    def _latex(self, p, **kwargs):
         if self in p._settings['symbol_names']:
             return p._settings['symbol_names'][self]
 
@@ -996,7 +1012,8 @@ class Symbol(AtomicExpr, NotIterable):
         result = p._deal_with_super_sub(name) if '\\' not in name else name
 
         if self.is_random:
-            return r'{\color{red} {%s}}' % result
+            color = kwargs.get('color', 'red')
+            return r'{\color{%s} {%s}}' % (color, result)
         
         if self.domain_assumed:
             return r"{\color{green} {%s}}" % result
@@ -1277,11 +1294,16 @@ class Symbol(AtomicExpr, NotIterable):
         from sympy.core.relational import Equal
         return Equal(self | y, self, **kwargs)
 
-    def as_boolean(self):
-        if self.is_random:
+    def as_boolean(self, **kwargs):
+        if self.is_random:            
+            if kwargs.get('surrogate'):
+                rhs = self.surrogate
+            else:
+                from sympy.stats.rv import pspace
+                rhs = pspace(self).symbol
+
             from sympy import Equal
-            from sympy.stats.rv import pspace
-            return Equal(self, pspace(self).symbol)
+            return Equal(self, rhs)
 
     def _subs(self, old, new, **hints):
         """Substitutes an expression old -> new. when self is a Symbol
@@ -1418,7 +1440,25 @@ class Symbol(AtomicExpr, NotIterable):
                 yield from value.find_path(cls, path, **kwargs)
                 path.pop()
                 
-            
+# performing other.indices in self.indices
+    def index_contains(self, other):
+        if other.is_Sliced:
+            base = other.base
+            if not base.is_Indexed:
+                return False
+
+            return self.index_contains(base)
+
+        if other.is_Indexed:
+            return other.base == self
+
+        if other.is_SlicedIndexed:
+            return other.base == self
+        
+        if other.is_Symbol:
+            return self == other
+
+
 class Dummy(Symbol):
     """Dummy symbols are each unique, even if they have the same name:
 
