@@ -192,8 +192,9 @@ class MinMaxBase(Expr, LatticeOp):
                 return ai
             cond = a in ai.args
             if not cond:
-                return ai.func(*[do(i, a) for i in ai.args],
-                    evaluate=False)
+                args = [do(i, a) for i in ai.args]
+                return ai.func(*args, 
+                               evaluate=len(args) < len(ai.args) or ai._has(ai.negated_type) and not any(arg._has(ai.negated_type) for arg in args))
             if isinstance(ai, cls):
                 return ai.func(*[do(i, a) for i in ai.args if i != a],
                     evaluate=False)
