@@ -24,7 +24,7 @@ def intlike(n):
 ############################ COMPLETE GAMMA FUNCTION ##########################
 ###############################################################################
 
-class gamma(Function):
+class Gamma(Function):
     r"""
     The gamma function
 
@@ -227,6 +227,7 @@ class gamma(Function):
 ################## LOWER and UPPER INCOMPLETE GAMMA FUNCTIONS #################
 ###############################################################################
 
+gamma = Gamma
 
 class lowergamma(Function):
     r"""
