@@ -34,6 +34,10 @@ class Integral(AddWithLimits):
     """Represents unevaluated integral."""    
     __slots__ = ('is_commutative',)
 
+    @property
+    def is_comparable(self):
+        return False
+
     def __new__(cls, function, *symbols, **assumptions):
         """Create an unevaluated integral.
 
@@ -535,7 +539,10 @@ class Integral(AddWithLimits):
                             function = ret
                             continue
 
-            if not isinstance(antideriv, Integral) and antideriv is not None:
+            final = hints.get('final', True)
+            # dotit may be iterated but floor terms making atan and acot
+            # continuous should only be added in the final round
+            if final and not isinstance(antideriv, Integral) and antideriv is not None:
                 for atan_term in antideriv.atoms(atan):
                     atan_arg = atan_term.args[0]
                     # Checking `atan_arg` to be linear combination of `tan` or `cot`
@@ -972,6 +979,9 @@ class Integral(AddWithLimits):
                     parts.append(coeff * h)
                     continue
 
+                if g.is_Probability:
+                    return
+
                 # Try risch again.
                 if risch is not False:
                     try:
@@ -993,7 +1003,7 @@ class Integral(AddWithLimits):
                             h = heurisch_wrapper(g, x, hints=[])
                         else:
                             h = heurisch_(g, x, hints=[])
-                    except PolynomialError:
+                    except (PolynomialError, AttributeError):
                         # XXX: this exception means there is a bug in the
                         # implementation of heuristic Risch integration
                         # algorithm.
@@ -1110,22 +1120,34 @@ class Integral(AddWithLimits):
         if len(limit) == 3:
             x, a, b = limit
             domain = self.expr.domain_nonzero(x)
-            from sympy.sets.sets import Interval
-            domain &= Interval(a, b)
+            
+            if a <= b:
+                from sympy.sets.sets import Interval
+                domain &= Interval(a, b, right_open=True)
+    
+                if self.expr.is_Piecewise:
+                    sgm = []
+                    for f, condition in self.expr.args:
+                        _domain = x.domain_conditioned(condition)
+                        _domain &= domain
+                        if f:
+                            if _domain.is_Interval:
+                                a, b = _domain.args
+                                if a <= b:
+                                    sgm.append(self.func(f, (x, a, b)).simplify())
+                                else:
+                                    sgm.append(self.func(f, (x, _domain)).simplify())
+                            else:
+                                sgm.append(self.func(f, (x, _domain)))
 
-            if isinstance(self.expr, Piecewise):
-                sgm = []
-                for f, condition in self.expr.args:
-                    if f == 0:
-                        continue
-                    _domain = x.domain_conditioned(condition) & domain
-                    sgm.append(self.func(f, (x, domain.start, domain.stop)).simplify())
-
-                return Add(*sgm)
-            try:
-                limit = x, domain.start, domain.stop
-            except AttributeError:
-                return self
+                        domain -= _domain
+    
+                    return Add(*sgm)
+                try:
+                    limit = x, domain.start, domain.stop
+                except AttributeError:
+                    return self
+                
         var = limit[0]
 
         function = self.expr
@@ -1137,50 +1159,30 @@ class Integral(AddWithLimits):
 
         if dependent == S.One:
             if len(limit) > 1:
-                x, a, b = limit
-                return self.expr * (b - a)
+                if len(limit) == 3:
+                    x, a, b = limit
+                    return self.expr * (b - a)
+                else:
+                    return self
             else:
                 return self.expr * var.dimension
+            
         return self.func(dependent, limit) * independent
 
     def _subs(self, old, new, **hints):
-        if self == old:
-            return new
+        this = self._subs_utility(old, new, **hints)
+        if this is not None:
+            return this
 
-        if len(self.limits) == 1:
+        if old in self.variables_set:
+            limits = []
+            for x, *ab in self.limits:
+                ab = [t._subs(old, new) for t in ab]
+                limits.append((x, *ab))
 
-            limit = self.limits[0]
-            if len(limit) == 1:
-                # deal with indefinite integrals
-                x = limit[0]
+            return self.func(self.expr, *limits)
 
-                function = self.expr.subs(old, new)
-                return self.func(function, (x,))
-
-            if old in self.variables_set:
-                limits = []
-                for x, *ab in self.limits:
-                    ab = [t._subs(old, new) for t in ab]
-                    limits.append((x, *ab))
-                
-                return self.func(self.expr, *limits)
-            function = self.expr._subs(old, new)
-
-            if len(limit) == 3:
-                x, a, b = limit
-                return self.func(function, (x, a.subs(old, new), b.subs(old, new))).simplify()
-            elif len(limit) == 2:
-                x, cond = limit
-                return self.func(function, (x, cond._subs(old, new))).simplify()
-            else:
-                x = limit[0]
-                return self.func(function, (x.subs(old, new)))
-        elif len(self.limits) == 0:
-            function = self.expr.subs(old, new)
-
-            return self.func(function, *self.limits)
-
-        return self
+        return AddWithLimits._subs(self, old, new, **hints)        
 
     def limits_subs(self, old, new, simplify=True):
         from sympy.solvers.solvers import solve
@@ -1528,7 +1530,17 @@ class Integral(AddWithLimits):
                 symbols.insert(0, r"\, d%s" % p._print(symbol))
 
         from sympy.printing.precedence import PRECEDENCE
-        return r"%s %s%s" % (tex, p.parenthesize(self.expr, PRECEDENCE["Mul"], strict=True), "".join(symbols))
+        expr = p.parenthesize(self.expr, PRECEDENCE["Mul"], strict=True)
+        symbols = "".join(symbols)
+        if self.expr.is_Integral:
+            return r"%s %s%s" % (tex, symbols, expr)
+        else:
+            return r"%s %s%s" % (tex, expr, symbols)
+
+    @classmethod
+    def identity(cls, self, **assumptions):
+        from sympy import ZeroMatrix
+        return ZeroMatrix(*self.shape)
 
 
 def integrate(*args, meijerg=None, conds='piecewise', risch=None, heurisch=None, manual=None, **kwargs):
