@@ -1481,7 +1481,7 @@ def eval_trigsubstitution(theta, func, rewritten, substep, restriction, integran
 @evaluates(DerivativeRule)
 def eval_derivativerule(integrand, symbol):
     # isinstance(integrand, Derivative) should be True
-    variable_count = list(integrand.variable_count)
+    variable_count = list(integrand.limits)
     for i, (var, count) in enumerate(variable_count):
         if var == symbol:
             variable_count[i] = (var, count - 1)
