@@ -36,8 +36,15 @@ class Softmax(Function):
             x = self.arg
             return exp(x[indices]) / ReducedSum(exp(x))
         if isinstance(indices, tuple):
-            i, *indices = indices
-            return self.func(self.arg[i])[indices]
+            i, *rest = indices
+            if isinstance(i, slice):
+                if len(self.shape) == 2:
+                    from sympy import Lamda, Basic
+                    j = self.generate_var(integer=True, excludes={j for j in indices if isinstance(j, Basic)})
+                    self = Lamda[j:self.shape[0]](self.func(self.arg[j]))
+                    return self[indices]
+            else:
+                return self.func(self.arg[i])[rest]
         else:
             return self.func(self.arg[indices])
  
