@@ -1,5 +1,5 @@
-from ..modules.module import Module
-from typing import Optional, List, Tuple, Union, TypeVar
+from .module import Module
+from typing import List, Tuple, Union, TypeVar
 from sympy.core.symbol import Symbol
 from sympy.keras.nn.modules.normalization import LayerNorm
 from itertools import repeat
@@ -35,7 +35,7 @@ def _reverse_repeat_tuple(t, n):
     return tuple(x for x in reversed(t) for _ in range(n))
 
 
-class _ConvNd(Module):
+class _ConvND(Module):
 
     __constants__ = ['stride', 'padding', 'dilation', 'groups',
                      'padding_mode', 'output_padding', 'in_channels',
@@ -70,7 +70,7 @@ class _ConvNd(Module):
                  padding_mode: str,
                  device=None,
                  dtype=None) -> None:
-        super(_ConvNd, self).__init__()
+        super(_ConvND, self).__init__()
         if in_channels % groups != 0:
             raise ValueError('in_channels must be divisible by groups')
         if out_channels % groups != 0:
@@ -143,12 +143,12 @@ class _ConvNd(Module):
         return s.format(**self.__dict__)
 
     def __setstate__(self, state):
-        super(_ConvNd, self).__setstate__(state)
+        super(_ConvND, self).__setstate__(state)
         if not hasattr(self, 'padding_mode'):
             self.padding_mode = 'zeros'
 
 
-class Conv1d(_ConvNd):
+class Conv1D(_ConvND):
     __doc__ = r"""Applies a 1D convolution over an input signal composed of several input
     planes.
 
@@ -229,7 +229,7 @@ class Conv1d(_ConvNd):
 
     Examples::
 
-        >>> m = nn.Conv1d(16, 33, 3, stride=2)
+        >>> m = nn.Conv1D(16, 33, 3, stride=2)
         >>> input = torch.randn(20, 16, 50)
         >>> output = m(input)
 
@@ -261,7 +261,7 @@ class Conv1d(_ConvNd):
         stride_ = _single(stride)
         padding_ = padding if isinstance(padding, str) else _single(padding)
         dilation_ = _single(dilation)
-        super(Conv1d, self).__init__(
+        super(Conv1D, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _single(0), groups, bias, padding_mode, **factory_kwargs)
 
@@ -270,14 +270,18 @@ class Conv1d(_ConvNd):
             return F.conv1d(F.pad(input, self._reversed_padding_repeated_twice, mode=self.padding_mode),
                             weight, bias, self.stride,
                             _single(0), self.dilation, self.groups)
-        return F.conv1d(input, weight, bias, self.stride,
+        conv1d = F.conv1d(input, weight, bias, self.stride,
                         self.padding, self.dilation, self.groups)
 
+        from sympy.core.singleton import S
+        S[input], S[weight], S[bias], S[self.stride], S[self.padding], S[self.dilation] = conv1d.of_conv1d()
+        return conv1d
+
     def forward(self, input):
         return self._conv_forward(input, self.W, self.b)
 
 
-class Conv2d(_ConvNd):
+class Conv2D(_ConvND):
     __doc__ = r"""Applies a 2D convolution over an input signal composed of several input
     planes.
 
@@ -371,11 +375,11 @@ class Conv2d(_ConvNd):
     Examples:
 
         >>> # With square kernels and equal stride
-        >>> m = nn.Conv2d(16, 33, 3, stride=2)
+        >>> m = nn.Conv2D(16, 33, 3, stride=2)
         >>> # non-square kernels and unequal stride and with padding
-        >>> m = nn.Conv2d(16, 33, (3, 5), stride=(2, 1), padding=(4, 2))
+        >>> m = nn.Conv2D(16, 33, (3, 5), stride=(2, 1), padding=(4, 2))
         >>> # non-square kernels and unequal stride and with padding and dilation
-        >>> m = nn.Conv2d(16, 33, (3, 5), stride=(2, 1), padding=(4, 2), dilation=(3, 1))
+        >>> m = nn.Conv2D(16, 33, (3, 5), stride=(2, 1), padding=(4, 2), dilation=(3, 1))
         >>> input = torch.randn(20, 16, 50, 100)
         >>> output = m(input)
 
@@ -405,7 +409,7 @@ class Conv2d(_ConvNd):
         stride_ = _pair(stride)
         padding_ = padding if isinstance(padding, str) else _pair(padding)
         dilation_ = _pair(dilation)
-        super(Conv2d, self).__init__(
+        super(Conv2D, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _pair(0), groups, bias, padding_mode, **factory_kwargs)
 
@@ -420,7 +424,7 @@ class Conv2d(_ConvNd):
     def forward(self, input):
         return self._conv_forward(input, self.W, self.b)
 
-class Conv3d(_ConvNd):
+class Conv3D(_ConvND):
     __doc__ = r"""Applies a 3D convolution over an input signal composed of several input
     planes.
 
@@ -507,9 +511,9 @@ class Conv3d(_ConvNd):
     Examples::
 
         >>> # With square kernels and equal stride
-        >>> m = nn.Conv3d(16, 33, 3, stride=2)
+        >>> m = nn.Conv3D(16, 33, 3, stride=2)
         >>> # non-square kernels and unequal stride and with padding
-        >>> m = nn.Conv3d(16, 33, (3, 5, 2), stride=(2, 1, 1), padding=(4, 2, 0))
+        >>> m = nn.Conv3D(16, 33, (3, 5, 2), stride=(2, 1, 1), padding=(4, 2, 0))
         >>> input = torch.randn(20, 16, 10, 50, 100)
         >>> output = m(input)
 
@@ -539,7 +543,7 @@ class Conv3d(_ConvNd):
         stride_ = _triple(stride)
         padding_ = padding if isinstance(padding, str) else _triple(padding)
         dilation_ = _triple(dilation)
-        super(Conv3d, self).__init__(
+        super(Conv3D, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
             False, _triple(0), groups, bias, padding_mode, **factory_kwargs)
 
@@ -572,7 +576,7 @@ class GLUCNN(Module):
         self.LayerNorm = LayerNorm(embed_size)
         padding = self.initial_offset((dilation,), (kernel_size,))
         
-        self.Conv1d = Conv1d(embed_size, embed_size * 2, kernel_size, dilation=dilation, padding=padding)
+        self.Conv1D = Conv1D(embed_size, embed_size * 2, kernel_size, dilation=dilation, padding=padding)
         
 # http://localhost/sympy/axiom.php?module=keras.eq_lamda_bool.imply.eq.conv1d
     def initial_offset(self, dilation, kernel_size, i=0):
@@ -582,8 +586,8 @@ class GLUCNN(Module):
         if mask is not None:
             inputs *= mask.unsqueeze(-1, inputs.shape[-1])
             
-        outputs = self.Conv1d(inputs)
-        filters = self.Conv1d.out_channels // 2
+        outputs = self.Conv1D(inputs)
+        filters = self.Conv1D.out_channels // 2
         
         return self.LayerNorm(inputs + outputs[:, :, :filters] * sigmoid(outputs[:, :, filters:]))
 
