@@ -60,7 +60,7 @@ class Invoker:
         
     def result(self, obj, simplify=True, evaluate=None):
         assumptions = {self.determine_assumptions(obj): self.source}
-        from sympy.core.sympify import _sympify        
+        from sympy.core.sympify import _sympify
         obj = _sympify(obj)
         for i in range(-1, -len(self.index) - 1, -1):
             this = self._objs[i - 1]
@@ -70,7 +70,7 @@ class Invoker:
             try:
                 args[index] = obj
             except TypeError:
-                assert this.is_And or this.is_Or
+                assert this.is_And or this.is_Or or this.is_Mul or this.is_Add
                 for j in index[::-1]:
                     del args[j]
                 args.append(obj)
@@ -193,7 +193,12 @@ class Invoker:
                             
                     _x = x.copy(domain=domain)
                     if x != _x:
-                        this = this._subs(x, _x).simplify()
+                        this = this._subs(x, _x)
+                        if this.is_ExprWithLimits and len(this.limits) > 1 and kwargs.get('simplify') is False:
+                            ...
+                        else:
+                            this = this.simplify()
+
                         reps.append((x, _x))
                         outer_context[x] = (_x, domain)
             
@@ -363,7 +368,7 @@ class Invoker:
             try:
                 index = parent.args.index(obj)
             except ValueError:
-                assert parent.is_And or parent.is_Or
+                assert parent.is_And or parent.is_Or or parent.is_Mul or parent.is_Add
                 index = tuple(parent.args.index(o) for o in obj)
                 obj = parent.func(*obj)
                 
