@@ -137,9 +137,9 @@ class MatrixShaping(MatrixRequired):
         return self._new(self.rows - 1, self.cols, entry)
 
     def _eval_row_insert(self, pos, other):
-        entries = list(self)
+        entries = [*self.args]
         insert_pos = pos * self.cols
-        entries[insert_pos:insert_pos] = list(other)
+        entries[insert_pos:insert_pos] = [*other.args]
         return self._new(self.rows + other.rows, self.cols, entries)
 
     def _eval_row_join(self, other):
