@@ -904,12 +904,9 @@ class MutableDenseMatrix(DenseMatrix):
 
         sympy.simplify.simplify.simplify
         """
-        return self
-        if isinstance(self._args, tuple):
-            return self.func(*(_simplify(self._args[i]) for i in range(len(self._args))), shape=self.shape)
-        for i in range(len(self._args)):
-            self._args[i] = _simplify(self._args[i], ratio=ratio, measure=measure, rational=rational, inverse=inverse)
-            
+        if all(arg.is_Zero for arg in self.args):
+            from sympy.matrices.expressions.matexpr import ZeroMatrix
+            return ZeroMatrix(*self.shape)
         return self
 
     def zip_row_op(self, i, k, f):
