@@ -59,6 +59,9 @@ class BlockMatrix(MatrixExpr):
     _eval_is_extended_positive = lambda self: self._eval_template_is_attr('is_extended_positive')
     _eval_is_extended_negative = lambda self: self._eval_template_is_attr('is_extended_negative')
 
+    def _eval_is_random(self):
+        return any(arg.is_random for arg in self.args)
+
     @property
     def dtype(self):
         dtype = None
@@ -347,7 +350,7 @@ class BlockMatrix(MatrixExpr):
                 self = self[j]
             return self
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         from sympy.concrete.products import Product
         if self.is_upper or self.is_lower:
             i = self.generate_var(integer=True)
