@@ -111,7 +111,8 @@ class MatrixExpr(Expr):
             return Identity(self.rows)
         elif other is S.One:
             return self
-        return MatPow(self, other).doit(deep=False)
+        from sympy.core.power import Pow
+        return Pow(self, other, evaluate=False)
 
     @_sympifyit('other', NotImplemented)
     @call_highest_priority('__pow__')
@@ -542,6 +543,9 @@ class MatrixExpr(Expr):
             return True
         return Eq(self, other, evaluate=False)
 
+    def _eval_is_random(self):
+        return any(arg.is_random for arg in self.args)
+
 
 def get_postprocessor(cls):
 
@@ -903,7 +907,7 @@ class Identity(ConstantMatrix, MatrixExpr):
                 return S.Zero
             return KroneckerDelta(i, j)
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         return S.One
 
     def _eval_is_extended_integer(self):
@@ -918,6 +922,11 @@ class Identity(ConstantMatrix, MatrixExpr):
     def doit(self, **hints):
         return self
     
+    def __rmul__(self, other): 
+        if isinstance(other, Basic) and len(other.shape) >= len(self.shape):
+            return other
+        return Mul(other, self)
+    
 
 def simplify_shape(shape):
     while shape:
@@ -957,13 +966,15 @@ class ZeroMatrix(ConstantMatrix, MatrixExpr):
     @_sympifyit('other', NotImplemented)
     @call_highest_priority('__rpow__')
     def __pow__(self, other):
-        if other != 1 and not self.is_square:
-            raise ShapeError("Power of non-square matrix %s" % self)
+        if other > 0:
+            return self
         if other == 0:
-            return Identity(self.rows)
-        if other < 1:
+            return OneMatrix(*self.shape)
+        if other < 0:
             raise ValueError("Matrix det == 0; not invertible.")
-        return self
+
+        from sympy import Pow
+        return Pow(self, other, evaluate=False)
 
     def _eval_transpose(self, axis=-1):
         if axis == self.default_axis:
@@ -975,7 +986,7 @@ class ZeroMatrix(ConstantMatrix, MatrixExpr):
     def _eval_trace(self):
         return S.Zero
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         return S.Zero
 
     def conjugate(self):
@@ -1066,6 +1077,13 @@ class ZeroMatrix(ConstantMatrix, MatrixExpr):
         shape.reverse()
         return ZeroMatrix(*shape, *self.expr.shape)
     
+    def _eval_exp(self):
+        return OneMatrix(*self.shape)
+    
+    def of(self, cls):
+        if cls.is_Zero or cls.is_ZeroMatrix:
+            return ()
+
 
 class GenericZeroMatrix(ZeroMatrix):
     """
@@ -1132,7 +1150,7 @@ class OneMatrix(ConstantMatrix, MatrixExpr):
     def _eval_trace(self):
         return S.One * self.rows
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         condition = Eq(self.shape[0], 1) & Eq(self.shape[1], 1)
         if condition == True:
             return S.One
@@ -1204,6 +1222,21 @@ class OneMatrix(ConstantMatrix, MatrixExpr):
         shape.reverse()
         return OneMatrix(*shape, *self.expr.shape)
     
+    @staticmethod
+    def expand_dims(self, shape, pivot):
+        consistent_shape = shape[:-pivot] 
+        extra_shape = shape[-pivot:]
+        consistent_shape_len = len(consistent_shape)
+#                   transpose matrix from (*consistent_shape, *extra_shape) to (*extra_shape, *consistent_shape)
+#                   (3, 4, 5, 6) => (5, 6, 3, 4)
+        axes = []
+        for j in range(len(extra_shape)):
+            for i in range(consistent_shape_len):
+                axes.append(consistent_shape_len + j - i)
+
+        from sympy import Transpose
+        return Transpose[tuple(axes)](self * OneMatrix(*consistent_shape, *extra_shape))
+
     
 def matrix_symbols(expr):
     return [sym for sym in expr.free_symbols if sym.is_Matrix]
@@ -1457,7 +1490,7 @@ class SwapMatrix(ElementaryMatrix, MatrixExpr):
     def j(self):
         return self.args[1]
 
-    def _eval_determinant(self):        
+    def _eval_determinant(self, **kwargs):        
         return 2 * KroneckerDelta(self.i, self.j) - 1
 
     def _eval_transpose(self, axis=-1):
@@ -1540,7 +1573,7 @@ class MulMatrix(ElementaryMatrix, MatrixExpr):
     def i(self):
         return self.args[-2]
     
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         return self.multiplier
 
     def _entry(self, i, j=None, **_):
@@ -1702,7 +1735,7 @@ class AddMatrix(ElementaryMatrix, MatrixExpr):
             return Lamda[j:self.n](piecewise)
         return piecewise            
 
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         return S.One
 
     @_sympifyit('other', NotImplemented)
@@ -1806,7 +1839,7 @@ class ShiftMatrix(ElementaryMatrix, MatrixExpr):
     def j(self):
         return self.args[-1]
     
-    def _eval_determinant(self):
+    def _eval_determinant(self, **kwargs):
         return (-1) ** (self.j - self.i)
 
     def _eval_transpose(self, axis=-1):
