@@ -1132,6 +1132,11 @@ class Range(Set):
         if self.max().is_extended_negative:
             return False
 
+    def _eval_is_empty(self):
+        start, stop = self.args
+        if start < stop:
+            return False
+
     def inverse(self):
         return
 
@@ -1268,6 +1273,24 @@ class Range(Set):
             self = self.copy(stop=S.Infinity)
             
         return self
+    
+    def conditionally_contains(self, ft):
+        a, b = self.args
+        if a <= ft < b:
+            return True
+
+        # given: a < b
+        # imply: t >= a and t < b
+        free_symbols = ft.free_symbols
+        if len(free_symbols) == 1:
+            t, = free_symbols
+            if t.is_integer:
+                cond = b - a > 0
+                if cond._has(t):
+                    _t = t.copy(domain=cond.domain_conditioned(t))
+                    if a._subs(t, _t) <= ft._subs(t, _t) < b._subs(t, _t):
+                        return True
+        
 
 converter[range] = Range
 
