@@ -252,8 +252,8 @@ def add_formulae(formulae):
 
     # TODO branching
     addb([], [S.Half, a, a + S.Half],
-         Matrix([fp(2*a - 1, z), fm(2*a, z)*z**(S(1)/4),
-                 fm(2*a - 1, z)*sqrt(z), fp(2*a, z)*z**(S(3)/4)])
+         Matrix((fp(2*a - 1, z), fm(2*a, z)*z**(S(1)/4),
+                 fm(2*a - 1, z)*sqrt(z), fp(2*a, z)*z**(S(3)/4)))
          * 2**(-2*a)*gamma(2*a)*z**((1 - 2*a)/4),
          Matrix([[1, 0, 0, 0]]),
          Matrix([[0, 1, 0, 0],
@@ -262,13 +262,12 @@ def add_formulae(formulae):
                  [z, 0, 0, 1 - a]]))
     x = 2*(4*z)**(S(1)/4)*exp_polar(I*pi/4)
     addb([], [a, a + S.Half, 2*a],
-         (2*sqrt(polar_lift(-1)*z))**(1 - 2*a)*gamma(2*a)**2 *
-         Matrix([besselj(2*a - 1, x)*besseli(2*a - 1, x),
+         Matrix((besselj(2*a - 1, x)*besseli(2*a - 1, x),
                  x*(besseli(2*a, x)*besselj(2*a - 1, x)
                     - besseli(2*a - 1, x)*besselj(2*a, x)),
                  x**2*besseli(2*a, x)*besselj(2*a, x),
                  x**3*(besseli(2*a, x)*besselj(2*a - 1, x)
-                       + besseli(2*a - 1, x)*besselj(2*a, x))]),
+                       + besseli(2*a - 1, x)*besselj(2*a, x)))) * (2*sqrt(polar_lift(-1)*z))**(1 - 2*a)*gamma(2*a)**2,
          Matrix([[1, 0, 0, 0]]),
          Matrix([[0, S(1)/4, 0, 0],
                  [0, (1 - 2*a)/2, -S(1)/2, 0],
@@ -286,11 +285,10 @@ def add_formulae(formulae):
                  0]]),
          Matrix([[1 - 2*a, 1, 0], [z/2, S.Half - a, S.Half], [0, z, 0]]))
     addb([S.Half], [b, 2 - b],
-         pi*(1 - b)/sin(pi*b)*
-         Matrix([besseli(1 - b, sqrt(z))*besseli(b - 1, sqrt(z)),
+         Matrix((besseli(1 - b, sqrt(z))*besseli(b - 1, sqrt(z)),
                  sqrt(z)*(besseli(-b, sqrt(z))*besseli(b - 1, sqrt(z))
                           + besseli(1 - b, sqrt(z))*besseli(b, sqrt(z))),
-                 besseli(-b, sqrt(z))*besseli(b, sqrt(z))]),
+                 besseli(-b, sqrt(z))*besseli(b, sqrt(z)))) * pi*(1 - b)/sin(pi*b),
          Matrix([[1, 0, 0]]),
          Matrix([[b - 1, S(1)/2, 0],
                  [z, 0, z],
@@ -342,11 +340,10 @@ def add_formulae(formulae):
     #     Formula.find_instantiations (creates 2!*3!*3**(2+3) ~ 3000
     #     instantiations ... But it's not too bad.
     addb([a, a + S.Half], [2*a, b, 2*a - b + 1],
-         gamma(b)*gamma(2*a - b + 1) * (sqrt(z)/2)**(1 - 2*a) *
-         Matrix([besseli(b - 1, sqrt(z))*besseli(2*a - b, sqrt(z)),
+         Matrix((besseli(b - 1, sqrt(z))*besseli(2*a - b, sqrt(z)),
                  sqrt(z)*besseli(b, sqrt(z))*besseli(2*a - b, sqrt(z)),
                  sqrt(z)*besseli(b - 1, sqrt(z))*besseli(2*a - b + 1, sqrt(z)),
-                 besseli(b, sqrt(z))*besseli(2*a - b + 1, sqrt(z))]),
+                 besseli(b, sqrt(z))*besseli(2*a - b + 1, sqrt(z)))) * gamma(b)*gamma(2*a - b + 1) * (sqrt(z)/2)**(1 - 2*a),
          Matrix([[1, 0, 0, 0]]),
          Matrix([[0, S(1)/2, S(1)/2, 0],
                  [z/2, 1 - b, 0, z/2],
@@ -374,7 +371,7 @@ def add_formulae(formulae):
     #      - exp(z)))
     # Manually tuned rule
     addb([1, 1, a], [2, 2, a+1],
-         Matrix([a*(log(-z) + expint(1, -z) + EulerGamma)/(z*(a**2 - 2*a + 1)),
+         Matrix([a*(log(-z) + expint(1, -z) + S.EulerGamma)/(z*(a**2 - 2*a + 1)),
                  a*(-z)**(-a)*(gamma(a) - uppergamma(a, -z))/(a - 1)**2,
                  a*exp(z)/(a**2 - 2*a + 1),
                  a/(z*(a**2 - 2*a + 1))]),
@@ -708,7 +705,11 @@ class Formula(object):
         self.C = Matrix([[1] + [0]*n])
 
         m = eye(n)
-        m = m.col_insert(0, zeros(n, 1))
+        if m.is_Matrix:
+            m = m.col_insert(0, zeros(n, 1))
+        else:
+            m = Matrix((zeros(n, 1), m))
+            
         l = poly.all_coeffs()[1:]
         l.reverse()
         self.M = m.row_insert(n, -Matrix([l])/poly.all_coeffs()[0])
@@ -733,7 +734,10 @@ class Formula(object):
 
     @property
     def closed_form(self):
-        return (self.C*self.B)[0]
+        expr = self.C*self.B
+        if expr.shape:
+            return expr[0]
+        return expr
 
     def find_instantiations(self, func):
         """
@@ -1976,12 +1980,18 @@ def _hyperexpand(func, z, ops0=[], z0=Dummy('z0'), premult=1, prem=0,
         from sympy import eye
         C = apply_operators(C, ops0,
                             make_derivative_operator(f.M.subs(f.z, z0)
-                                         + prem*eye(f.M.shape[0]), z0))
+                                         + prem*eye(f.M.shape[0] if f.M.shape else 1), z0))
 
         if premult == 1:
-            C = C.applyfunc(make_simp(z0))
+            if C.shape:
+                C = C.applyfunc(make_simp(z0))
+            else:
+                C = make_simp(z0)(C)
         r = C*f.B.subs(f.z, z0)*premult
-        res = r[0].subs(z0, z)
+        if r.shape:
+            res = r[0].subs(z0, z)
+        else:
+            res = r.subs(z0, z)
         if rewrite:
             res = res.rewrite(rewrite)
         return res
