@@ -20,7 +20,7 @@ from sympy.solvers.inequalities import reduce_rational_inequalities
 from sympy.core.sympify import _sympify
 from sympy.external import import_module
 from sympy.stats.rv import (RandomDomain, SingleDomain, ConditionalDomain, is_random,
-        ProductDomain, PSpace, SinglePSpace, random_symbols, NamedArgsMixin,
+        ProductDomain, PSpace, SinglePSpace, NamedArgsMixin,
     Distribution)
 
 
@@ -581,7 +581,7 @@ class ContinuousPSpace(PSpace):
             return result if not cond_inv else S.One - result
 
     def where(self, condition):
-        rvs = frozenset(random_symbols(condition))
+        rvs = frozenset(condition.random_symbols)
         if not (len(rvs) == 1 and rvs.issubset(self.values)):
             raise NotImplementedError("Multiple continuous random variables not supported")
         rv = tuple(rvs)[0]
@@ -698,7 +698,7 @@ class SingleContinuousPSpace(ContinuousPSpace, SinglePSpace):
 def _reduce_inequalities(conditions, var, **kwargs):
     try:
         return reduce_rational_inequalities(conditions, var, **kwargs)
-    except PolynomialError:
+    except (PolynomialError, AssertionError):
         raise ValueError("Reduction of condition failed %s\n" % conditions[0])
 
 
