@@ -5,7 +5,7 @@ from sympy import (sympify, S, pi, sqrt, exp, Lambda, Indexed, besselk, gamma, I
 from sympy.matrices import ImmutableMatrix, MatrixSymbol
 from sympy.matrices.expressions.determinant import det
 from sympy.stats.joint_rv import JointDistribution, JointPSpace, MarginalDistribution
-from sympy.stats.rv import _value_check, random_symbols
+from sympy.stats.rv import _value_check
 
 __all__ = ['JointRV',
 'MultivariateNormal',
@@ -115,7 +115,7 @@ def JointRV(symbol, pdf, _set=None):
     pdf = Lambda(syms, pdf)
     dist = JointDistributionHandmade(pdf, _set)
     jrv = JointPSpace(symbol, dist).value
-    rvs = random_symbols(pdf)
+    rvs = pdf.random_symbols
     if len(rvs) != 0:
         dist = MarginalDistribution(dist, (jrv,))
         return JointPSpace(symbol, dist).value
