@@ -136,8 +136,6 @@ class ConditionalDomain(RandomDomain):
     """
 
     def __new__(cls, fulldomain, condition):
-#         condition = condition.xreplace({rs: rs.symbol
-#             for rs in random_symbols(condition)})
         return Basic.__new__(cls, fulldomain, condition)
 
     @property
@@ -432,20 +430,11 @@ class IndependentProductPSpace(ProductPSpace):
         symbols = set().union(*[sp.symbols for sp in spaces])
 
         # Overlapping symbols
-        from sympy.stats.joint_rv import MarginalDistribution
-        from sympy.stats.compound_rv import CompoundDistribution
-        if len(symbols) < sum(len(space.symbols) for space in spaces if not
-         isinstance(space.distribution, (
-            CompoundDistribution, MarginalDistribution))):
-            raise ValueError("Overlapping Random Variables")
-
         if all(space.is_Finite for space in spaces):
             from sympy.stats.frv import ProductFinitePSpace
             cls = ProductFinitePSpace
 
-        obj = Basic.__new__(cls, *FiniteSet(*spaces))
-
-        return obj
+        return Basic.__new__(cls, *FiniteSet(*spaces))
 
     @property
     def pdf(self):
@@ -506,7 +495,7 @@ class IndependentProductPSpace(ProductPSpace):
         elif isinstance(condition, Or):  # they are independent
             return Add(*[self.probability(arg) for arg in condition.args])
         expr = condition.lhs - condition.rhs
-        rvs = random_symbols(expr)
+        rvs = expr.random_symbols
         dens = self.compute_density(expr)
         if any([pspace(rv).is_Continuous for rv in rvs]):
             from sympy.stats.crv import SingleContinuousPSpace
@@ -531,13 +520,13 @@ class IndependentProductPSpace(ProductPSpace):
         return result if not cond_inv else S.One - result
 
     def compute_density(self, expr, **kwargs):
-        rvs = random_symbols(expr)
+        rvs = expr.random_symbols
         
         reps = self.values2symbols()
         for var in rvs: 
             expr = expr._subs(var, reps[var])
 
-        assert not random_symbols(expr)
+        assert not expr.random_symbols
 
         if any(pspace(rv).is_Continuous for rv in rvs):
             z = Dummy('z', real=True)
@@ -551,7 +540,7 @@ class IndependentProductPSpace(ProductPSpace):
         raise ValueError("CDF not well defined on multivariate expressions")
 
     def conditional_space(self, condition, normalize=True, **kwargs):
-        rvs = random_symbols(condition)
+        rvs = condition.random_symbols
         condition = condition.xreplace({rv: rv.symbol for rv in self.values})
         if any([pspace(rv).is_Continuous for rv in rvs]):
             from sympy.stats.crv import (ConditionalContinuousDomain,
@@ -649,30 +638,6 @@ class ProductDomain(RandomDomain):
         return And(*[domain.as_boolean() for domain in self.domains])
 
 
-def random_symbols(expr):
-    
-    """
-    Returns all RandomSymbols within a SymPy Expression.
-    """
-
-    def preorder_traversal(self):
-        if isinstance(self, Basic): 
-            if self.is_symbol:
-                if self.is_random:
-                    yield self
-            elif self.is_PDF:
-                ...
-            else:
-                if hasattr(self, '_argset'):
-                    args = self._argset
-                else:
-                    args = self.args
-    
-                for arg in args:
-                    yield from preorder_traversal(arg)
-                       
-    return [*preorder_traversal(expr)]
-
 
 def contain_random_symbols(expr):
     """
@@ -716,7 +681,7 @@ def pspace(expr):
             from sympy.stats.crv import SingleContinuousPSpace
             return SingleContinuousPSpace(expr)
 
-    rvs = random_symbols(expr)
+    rvs = expr.random_symbols
     if not rvs:
         raise ValueError("Expression containing Random Variable expected, not %s" % (expr))
     # If only one space present
@@ -808,33 +773,6 @@ def given(expr, condition, **kwargs):
     if condition.is_symbol:
         condition = Eq(condition, pspace(condition).symbol)
 
-#     condsymbols = random_symbols(condition)
-#     if condition.is_Equal and len(condsymbols) == 1 and not isinstance(pspace(expr).domain, ConditionalDomain):
-#         rv = tuple(condsymbols)[0]
-# 
-#         results = solveset(condition, rv)
-#         if isinstance(results, Intersection) and Reals in results.args:
-#             results = list(results.args[1])
-# 
-#         sums = 0
-#         for res in results:
-#             temp = expr.subs(rv, res)
-#             if temp == True:
-#                 return True
-#             if temp != False:
-#                 # XXX: This seems nonsensical but preserves existing behaviour
-#                 # after the change that Relational is no longer a subclass of
-#                 # Expr. Here expr is sometimes Relational and sometimes Expr
-#                 # but we are trying to add them with +=. This needs to be
-#                 # fixed somehow.
-#                 if sums == 0 and isinstance(expr, Relational):
-#                     sums = expr.subs(rv, res)
-#                 else:
-#                     sums += expr.subs(rv, res)
-#         if sums == 0:
-#             return False
-#         return sums
-
     # Get full probability space of both the expression and the condition
     fullspace = pspace(Tuple(expr, condition))
     # Build new space given the condition
@@ -963,7 +901,7 @@ class PDF(Basic):
         if condition is not None:
             # Recompute on new conditional expr
             expr = given(expr, condition, **kwargs)
-        if not random_symbols(expr):
+        if not expr.random_symbols:
             x = Symbol('x', real=True)
             return Lambda(x, DiracDelta(x - expr))
         ps = pspace(expr)
@@ -1578,7 +1516,7 @@ def pspace_independent(a, b):
     a_symbols = set(pspace_a.symbols)
     b_symbols = set(pspace_b.symbols)
 
-    if len(set(random_symbols(a)).intersection(random_symbols(b))) != 0:
+    if len(set(a.random_symbols).intersection(b.random_symbols)) != 0:
         return False
 
     if len(a_symbols.intersection(b_symbols)) == 0:
@@ -1592,7 +1530,7 @@ def rv_subs(expr, symbols=None):
     If symbols keyword is given restrict the swap to only the symbols listed.
     """
     if symbols is None:
-        symbols = random_symbols(expr)
+        symbols = expr.random_symbols
     if not symbols:
         return expr
     swapdict = {rv: rv.symbol for rv in symbols}
