@@ -15,7 +15,7 @@ from sympy.logic.boolalg import Boolean
 from sympy.utilities.exceptions import SymPyDeprecationWarning
 from sympy.stats.joint_rv import JointDistribution
 from sympy.stats.joint_rv_types import JointDistributionHandmade
-from sympy.stats.rv import (RandomIndexedSymbol, random_symbols, RandomSymbol,
+from sympy.stats.rv import (RandomIndexedSymbol, RandomSymbol,
                             _symbol_converter, _value_check, pspace, given,
                            dependent, is_random, sample_iter)
 from sympy.stats.stochastic_process import StochasticPSpace
@@ -665,7 +665,7 @@ class MarkovProcess(StochasticProcess):
         if check:
             return Expectation(expr, condition)
 
-        rvs = random_symbols(expr)
+        rvs = expr.random_symbols
         if isinstance(expr, Expr) and isinstance(condition, Eq) \
             and len(rvs) == 1:
             # handle queries similar to E(f(X[i]), Eq(X[i-m], <some-state>))
@@ -1178,7 +1178,7 @@ class _SubstituteRV:
 
         """
 
-        rvs_expr = random_symbols(expr)
+        rvs_expr = expr.random_symbols
         if len(rvs_expr) != 0:
             swapdict_expr = {}
             for rv in rvs_expr:
@@ -1186,7 +1186,7 @@ class _SubstituteRV:
                     newrv = rv.pspace.process.simple_rv(rv) # substitute with equivalent simple rv
                     swapdict_expr[rv] = newrv
             expr = expr.subs(swapdict_expr)
-        rvs_cond = random_symbols(condition)
+        rvs_cond = condition.random_symbols
         if len(rvs_cond)!=0:
             swapdict_cond = {}
             for rv in rvs_cond:
@@ -1253,7 +1253,7 @@ class _SubstituteRV:
         new_condition, new_givencondition = self._rvindexed_subs(condition, given_condition)
 
         if isinstance(new_givencondition, RandomSymbol):
-            condrv = random_symbols(new_condition)
+            condrv = new_condition.random_symbols
             if len(condrv) == 1 and condrv[0] == new_givencondition:
                 return BernoulliDistribution(self._probability(new_condition), 0, 1)
 
