@@ -2,7 +2,7 @@ import itertools
 
 from sympy import (Expr, Add, Mul, S, Integral, Eq, Sum, Symbol,
                     expand as _expand, Not)
-from sympy.core.compatibility import default_sort_key, ordered
+from sympy.core.compatibility import default_sort_key
 from sympy.core.parameters import global_parameters
 from sympy.core.sympify import _sympify
 from sympy.core.relational import Equal, Unequal
@@ -10,9 +10,13 @@ from sympy.logic.boolalg import BooleanFunction, And
 from sympy.stats import variance, covariance, rv
 from sympy.stats.rv import (RandomSymbol, pspace, dependent,
                             given, sampling_E, RandomIndexedSymbol, is_random,
-                            PSpace, sampling_P, random_symbols)
+                            PSpace, sampling_P)
 from sympy.concrete.expr_with_limits import ExprWithLimits
 from sympy.core.cache import cacheit
+from sympy.core.expr import AtomicExpr
+from sympy.core.containers import Tuple
+from _collections import defaultdict
+import std
 
 __all__ = ['Probability', 'Expectation', 'Variance', 'Covariance']
 
@@ -101,6 +105,9 @@ class Conditioned(Expr):
         return True
 
     def __new__(cls, prob, given, evaluate=True):
+        if prob.is_BooleanAtom:
+            return prob
+
         assert prob.is_random, prob
         assert given.is_random, given
         
@@ -115,6 +122,13 @@ class Conditioned(Expr):
             else:
                 if given in prob._argset:
                     prob = prob.func(*prob._argset - {given})
+                elif given.is_Equal:
+                    lhs, rhs = given.args
+                    if rhs.is_Surrogate:
+                        _given = Equal(lhs, rhs.arg.var, evaluate=False)
+                        if _given in prob._argset:
+                            prob = prob.func(*prob._argset - {_given})
+                        
         return Expr.__new__(cls, prob, given)
 
     def doit(self, **hints):
@@ -129,7 +143,8 @@ class Conditioned(Expr):
         return self.lhs.shape
     
     def _latex(self, p):
-        return r"%s \mid %s" % (p._print(self.lhs), p._print(self.rhs))
+        # return r"%s \middle \vert %s" % (p._print(self.lhs), p._print(self.rhs))
+        return r"%s \mathrel{\bigg|} %s" % (p._print(self.lhs), p._print(self.rhs))
 
     def _eval_is_real(self):
         return self.lhs.is_real
@@ -163,6 +178,186 @@ class Conditioned(Expr):
         if not self.is_bool:
             return self.func(self.lhs.as_boolean(), self.rhs)
 
+    def yield_random_symbols(self):
+        yield from self.lhs.yield_random_symbols()
+
+    def simplify(self, deep=False):
+        lhs, rhs = self.args
+        if lhs.is_And:
+            _argset = lhs._argset
+            if rhs in _argset:
+                lhs = And(*{*_argset} - {rhs})
+                return self.func(lhs, rhs, evaluate=False)
+            
+            elif rhs.is_Equal:
+                x, x_var = rhs.args
+                if x_var.is_Surrogate:
+                    rhs = Equal(x, x_var.arg.var, evaluate=False)
+                    if rhs in _argset:
+                        lhs = And(*{*_argset} - {rhs})
+                        return self.func(lhs, rhs, evaluate=False)
+                    
+        return Expr.simplify(self, deep)
+
+    def _try_to_substitute(self, other, new):
+        if not other.is_Conditioned:
+            return
+        
+        if self.lhs != other.lhs:
+            return
+        
+        rhs = self.rhs
+        _rhs = other.rhs
+        if rhs.is_And and _rhs.is_And:
+            rhs = rhs.args
+            _rhs = _rhs.args
+            if len(rhs) == len(_rhs):
+                conds = []
+                for eq, _eq in zip(rhs, _rhs):
+                    if new := eq._try_to_substitute(_eq, new):
+                        ...
+                    else:
+                        return
+
+                return new
+            
+        elif rhs.is_Equal and _rhs.is_Equal:
+            if new := rhs._try_to_substitute(_rhs, new):
+                return new
+    
+    def _subs(self, old, new, **hints):
+        if _new := self._try_to_substitute(old, new):
+            return _new
+        
+        return Expr._subs(self, old, new)
+
+    def __iter__(self):
+        raise TypeError
+    
+    def __getitem__(self, indices, **kw_args):
+        lhs, rhs = self.args
+        return self.func(lhs[indices], rhs, evaluate=False)
+
+
+class Surrogate(AtomicExpr):
+    is_symbol = True
+    is_comparable = False
+    
+    def __new__(cls, symbol):
+        obj = Expr.__new__(cls, symbol)
+        return obj
+    
+    @property
+    def var(self):
+        return self.arg.var
+    
+    @property
+    def dtype(self):
+        return self.arg.dtype
+    
+    @cacheit
+    def _eval_shape(self):
+        return self.arg.shape
+    
+    def __contains__(self, other):
+        return other in self.symbol
+    
+# precondition, self and other are structurally equal!
+    def _dummy_eq(self, other):
+        return self.arg == other
+
+    def structurally_equal(self, other):
+        return self.arg.structurally_equal(other)
+
+    def __iter__(self):
+        raise TypeError
+      
+    def __getitem__(self, indices):
+        return Surrogate(self.arg.__getitem__(indices))
+    
+    def _sympystr(self, p): 
+        return self.arg._sympystr(p)
+    
+    def _latex(self, p):
+        return self.arg._latex(p, color='magenta')
+    
+    def _eval_is_extended_positive(self):
+        return self.arg._eval_is_extended_positive()
+    
+    def _eval_is_extended_negative(self):
+        return self.arg._eval_is_extended_negative()
+    
+    def _eval_is_nonzero(self):
+        return self.arg._eval_is_nonzero()
+    
+    def _eval_is_zero(self):
+        return self.arg._eval_is_zero()
+    
+    def _eval_is_finite(self):
+        return self.arg._eval_is_finite()
+    
+    def _eval_is_extended_integer(self):
+        return self.arg._eval_is_extended_integer()
+    
+    def _eval_is_super_integer(self):
+        return self.arg._eval_is_super_integer()
+    
+    def _eval_is_extended_rational(self):
+        return self.arg._eval_is_extended_rational()
+    
+    def _eval_is_hyper_rational(self):
+        return self.arg._eval_is_hyper_rational()
+    
+    def _eval_is_super_rational(self):
+        return self.arg._eval_is_super_rational()
+    
+    def _eval_is_extended_real(self):
+        return self.arg._eval_is_extended_real()
+    
+    def _eval_is_hyper_real(self):
+        return self.arg._eval_is_hyper_real()
+    
+    def _eval_is_super_real(self):
+        return self.arg._eval_is_super_real()
+    
+    def _eval_is_extended_complex(self):
+        return self.arg._eval_is_extended_complex()
+    
+    def _eval_is_hyper_complex(self):
+        return self.arg._eval_is_hyper_complex()
+    
+    def _eval_is_algebraic(self):
+        return self.arg._eval_is_algebraic()
+    
+    def _eval_is_hermitian(self):
+        return self.arg._eval_is_hermitian()
+    
+    def _eval_is_imaginary(self):
+        return self.arg._eval_is_imaginary()
+    
+    def _eval_is_random(self):
+        return self.arg.is_random
+    
+    @property
+    def distribution(self):
+        return self.arg.distribution
+
+    def as_boolean(self):
+        from sympy import Equal
+        return Equal(self.arg, self, evaluate=False)
+
+    def __and__(self, other):
+        """Overloading for & operator"""
+        if other.is_random:
+            if not other.is_bool:
+                other = other.as_boolean()
+            return self.as_boolean() & other 
+
+        return super(Surrogate, self).__and__(other)
+
+    def copy(self, **kwargs):
+        return self.arg.copy(**kwargs)
+
 
 # when X is a discrete random variable, define P(X) = P(X=x), which is necessarily nomore than 1, Sum[X](P(X)) = 1
 # when X is a continuous random variable, define P(X) = P(X=x) = PDF(X)(x) which is possibly greater than 1, Integral[X](P(X)) = 1
@@ -198,9 +393,13 @@ class Probability(Expr):
     def _eval_domain_defined(self, x, **_):        
         return self.arg.domain_defined(x)            
 
-    def marginalize(self, given):
-        assert given.is_symbol and given.is_random        
-        return self.func.marginalize_condition(self.arg, given)
+    def marginalize(self, given, *rest):
+        assert given.is_symbol and given.is_random
+        prob = self.func.marginalize_condition(self.arg, given)
+        if rest:
+            prob = prob.marginalize(*rest)
+            
+        return prob
                 
     @classmethod    
     def marginalize_condition(cls, condition, given):        
@@ -261,30 +460,58 @@ class Probability(Expr):
     
     def __new__(cls, *prob, given=None, evaluate=False): 
         booleans = []
-        limits = ()
+        limits = []
         for arg in prob:
-            if isinstance(arg, tuple):
-                limits = arg
+            if isinstance(arg, (tuple, Tuple)):
+                limits.append(Tuple(*arg))
                 continue
-
+            
+            if arg.is_BooleanTrue:
+                continue
+            
+            if arg.is_BooleanFalse:
+                return S.Zero
+            
             assert arg.is_random
             if arg.is_symbol:
                 booleans.append(Equal(arg, pspace(arg).symbol))
             elif arg.is_Conditioned:
                 lhs, rhs = arg.args
                 if lhs.is_symbol:
-                    booleans.append(arg.func(Equal(lhs, pspace(lhs).symbol), rhs))
+                    if lhs.is_Surrogate:
+                        booleans.append(arg.func(Equal(lhs.arg, lhs), rhs))
+                    else:
+                        booleans.append(arg.func(Equal(lhs, pspace(lhs).symbol), rhs))
                 else:
                     booleans.append(arg)
             else:
                 assert arg.is_Boolean, type(arg)
                 booleans.append(arg)
+
+        if not booleans:
+            return S.One
+
         expr = And(*booleans)
         if given is not None:
             expr = rv.given(expr, given)
-        obj = Expr.__new__(cls, expr, *limits)
-        return obj
 
+        vars = [v for v in expr.random_symbols if not v.is_Surrogate]
+        std.deleteIndices(limits, lambda limits, i : cls.is_redundant(limits, i, vars))
+
+        return Expr.__new__(cls, expr, *limits)
+
+    @classmethod
+    def is_redundant(cls, limits, i, vars):
+        limit = limits[i]
+        if len(limit) > 1:
+            x, w = limit
+            if x.has(*vars):
+                if x.is_Indexed or x.is_Sliced:
+                    if not any(i.is_random for i in x.indices):
+                        limits[i] = Tuple(x.base, w)
+            else:
+                return True
+        
     def doit(self, **hints):
         condition = self.args[0]
         given_condition = None
@@ -298,7 +525,7 @@ class Probability(Expr):
             return pspace(condition).probability(condition, given_condition, evaluate=for_rewrite)
 
         if isinstance(given_condition, RandomSymbol):
-            condrv = random_symbols(condition)
+            condrv = condition.random_symbols
             if len(condrv) == 1 and condrv[0] == given_condition:
                 from sympy.stats.frv_types import BernoulliDistribution
                 return BernoulliDistribution(self.func(condition).doit(**hints), 0, 1)
@@ -314,7 +541,7 @@ class Probability(Expr):
             return S.Zero
         
         if not condition.is_Relational:
-            raise ValueError("%s is not a relational or combination of relationals" % (condition))
+            return self
         
         if condition is S.true:
             return S.One
@@ -330,7 +557,10 @@ class Probability(Expr):
         if ps  == PSpace():
             return Probability(condition, given_condition)
 
-        return ps.probability(condition).simplify()
+        try:
+            return ps.probability(condition).simplify()
+        except:
+            return self
 
     def _eval_rewrite_as_Integral(self, arg, condition=None, **kwargs):
         return self.func(arg, condition=condition).doit(for_rewrite=True)
@@ -340,19 +570,50 @@ class Probability(Expr):
     def evaluate_integral(self):
         return self.rewrite(Integral).doit()
 
+    def normalize_limits(self, limits):
+        for i, limit in enumerate(limits):
+            if len(limit) > 1:
+                limits[i] = (limit[-1],)
+        return limits
+
+    @cacheit
+    def _eval_is_pdf(self):
+        expr = self.arg
+        random_symbols = {v for v in expr.random_symbols if not v.is_Surrogate}
+
+        if expr.is_Conditioned:
+            expr = expr.lhs
+
+        conds = expr.args if expr.is_And else [expr]
+        if all(cond.is_Equal for cond in conds): 
+            for cond in conds:
+                lhs = cond.args[0]
+                if lhs in random_symbols and not lhs.is_integer and lhs.is_symbol:
+                    return True
+        
+    @property
+    def is_pdf(self):
+        return self._eval_is_pdf()
+
     def _latex(self, p):
         expr, *limits = self.args
+        symbol_P = 'p' if self.is_pdf else 'P'
         if limits:
-            return r'\mathbb{P}_{%s}%s' % (','.join([p._print(s) for s in limits]), p._print(self.args[:1]))
+            limits = self.normalize_limits(limits)
+            return r'\mathbb{%s}_{%s}\left(%s\right)' % (
+                symbol_P,
+                ','.join([p._print(s) for s, *_ in limits]), 
+                p._print(expr))
         else:
-            return r'\mathbb{P}%s' % p._print(self.args)
+            return r'\mathbb{%s}\left(%s\right)' % (symbol_P, p._print(expr))
 
     def _sympystr(self, p):
         expr, *limits = self.args
         if limits:
-            return 'P[%s](%s)' % (','.join([p._print(s) for s in limits]), p._print(expr))
+            limits = self.normalize_limits(limits)
+            return 'P[%s](%s)' % (','.join([p._print(s) for s, *_ in limits]), p._print(expr))
         else:
-            return 'P(%s)' % p._print(self.args)
+            return 'P(%s)' % p._print(expr)
 
     def domain_definition(self):        
         return self.arg.domain_definition()
@@ -361,6 +622,18 @@ class Probability(Expr):
         if old.is_Conditioned and not old.lhs.is_bool or new.is_Conditioned and not new.lhs.is_bool:
             old = old.as_boolean()
             new = new.as_boolean()
+            
+        else:
+            random_symbols = self.args[0].random_symbols
+            if old in random_symbols:
+                if not old.is_Surrogate:
+                    old = old.surrogate
+                    if old in random_symbols:
+                        if new.is_random:
+                            new = new.surrogate
+                    else:
+                        return self
+
         return Expr._subs(self, old, new, **hints)
 
     @classmethod
@@ -368,13 +641,40 @@ class Probability(Expr):
         """
         precondition: self.lhs is a Probability object!
         """
-        if rhs.is_Probability:                
+        if lhs.is_Probability and len(lhs.args) == 1 and rhs.is_Probability and len(rhs.args) == 1:
             lhs = self.lhs.arg.simplify_condition_on_random_variable()
             if lhs is not self.lhs.arg:
                 rhs = self.rhs.arg.simplify_condition_on_random_variable()
                 if rhs is not self.rhs.arg:
                     return self.func(lhs, rhs)
-                
+
+    def yield_random_symbols(self):
+        expr, *limits = self.args
+        
+        if expr.is_Conditioned:
+            lhs, rhs = expr.args
+            for v in lhs.yield_random_symbols():
+                if v.is_Surrogate:
+                    yield v.arg
+
+            for v in rhs.yield_random_symbols():
+                if v.is_Surrogate:
+                    yield v.arg
+        else:
+            for v in expr.yield_random_symbols():
+                if v.is_Surrogate:
+                    yield v.arg
+
+        for x, *ab in limits:
+            for w in ab:
+                yield from w.yield_random_symbols()
+
+    def _eval_is_random(self):
+        for v in self.arg.random_symbols:
+            if v.is_Surrogate:
+                return True
+    
+
 class Expectation(ExprWithLimits):
     """
     Symbolic expression for the expectation.
@@ -444,18 +744,83 @@ class Expectation(ExprWithLimits):
     mu + Expectation(Expectation(Y + Expectation(2*X)))
 
     """
-    is_Expectation = True
-    is_complex = True
-    
+
     def __new__(cls, expr, *limits, **kwargs):
+        # only the last limit can be the weight limit
         expr = _sympify(expr)
-        if expr.is_Matrix:
-            from sympy.stats.symbolic_multivariate_probability import ExpectationMatrix
-            return ExpectationMatrix(expr)
         if not expr.is_random:
             return expr
+        
+        weights = []
+        
+        def deleteIndices(limit):
+            nonlocal weights 
+            x, *ab = limit
+            if not x.is_random:
+                weights += [x, *ab]
+                return True
+
+        if (ret := std.deleteIndices(limits, deleteIndices)) is not None:
+            limits = ret
+                
+        random_symbols = expr.random_symbols
+        if limits:
+            varsAltered = defaultdict(set)
+            for x, *ab in limits:
+                if ab or not x.shape:
+                    continue
+
+                for xk in random_symbols:
+                    if not xk.is_Indexed:
+                        continue
+                     
+                    if xk.base == x:
+                        varsAltered[x].add((xk,))
+                    elif x.is_Sliced and x.base == xk.base:
+                        if len(x.args) <= len(xk.args):
+                            for (start, stop), k in zip(x.indices, xk.indices):
+                                if start <= k < stop:
+                                    varsAltered[x].add((xk,))
+                
+            if varsAltered:
+                limits = [*limits]
+                for x, xks in varsAltered.items():
+                    i = std.indexFor(limits, lambda limit: limit[0] == x)
+                    std.splice(limits, i, 1, *xks)
+
+        else:
+            limits = [(x,) for x in random_symbols]
+            
+        if weights:
+            limits.append(tuple(weights))
+
+        if (given := kwargs.get('given')) is not None:
+            if given.is_symbol:
+                given = Equal(given, given.var)
+            expr = Conditioned(expr, given)
+
         return ExprWithLimits.__new__(cls, expr, *limits)
+
+    @classmethod
+    def unnest(cls, expr, limits, symbols, **assumptions):
+        if not limits and symbols:
+            return expr.copy(**assumptions)
+        return Expr.__new__(cls, expr, *limits, **assumptions)
+
+    def _eval_is_complex(self):
+        return True
+
+    def _has_weights(self):
+        return not self.args[-1][0].is_random
+
+    @property
+    def limits(self):
+        return self.args[1:-1] if self._has_weights() else self.args[1:]
     
+    @property
+    def weights(self):
+        return self.args[-1] if self._has_weights() else Tuple()
+
     @property
     def expr(self):
         return self.args[0]
@@ -470,19 +835,16 @@ class Expectation(ExprWithLimits):
 
     def expand(self, **hints):
         expr = self.args[0]
-        condition = self._condition
 
         if not is_random(expr):
             return expr
 
         if isinstance(expr, Add):
-            return Add.fromiter(Expectation(a, condition=condition).expand()
-                    for a in expr.args)
+            return Add.fromiter(Expectation(a).expand() for a in expr.args)
 
         expand_expr = _expand(expr)
         if isinstance(expand_expr, Add):
-            return Add.fromiter(Expectation(a, condition=condition).expand()
-                    for a in expand_expr.args)
+            return Add.fromiter(Expectation(a).expand() for a in expand_expr.args)
 
         elif isinstance(expr, Mul):
             rv = []
@@ -492,7 +854,7 @@ class Expectation(ExprWithLimits):
                     rv.append(a)
                 else:
                     nonrv.append(a)
-            return Mul.fromiter(nonrv) * Expectation(Mul.fromiter(rv), condition=condition)
+            return Mul.fromiter(nonrv) * Expectation(Mul.fromiter(rv))
 
         return self
 
@@ -504,7 +866,7 @@ class Expectation(ExprWithLimits):
                 _x = x.copy(distribution=distribution)
                 expr = function._subs(x, _x)
                 if expr.is_Conditioned:
-                    expr = expr.doit()                    
+                    expr = expr.doit()
                 if not expr.is_random:  # expr isn't random?
                     return expr
                 ps = pspace(expr)
@@ -591,36 +953,152 @@ class Expectation(ExprWithLimits):
     def _latex(self, p):
         tex = r'\mathop{\mathbb{E}}'
         limits = self.limits
+
+        if weights := self.weights:
+            weights_str = ','.join([p._print(w) for w in weights])
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return r"%s \sim %s" % (p._print(limit[0]), weights_str)
+                x, cond = limit
+                return r"%s \sim %s \mid %s" % (p._print(x), p._print(cond), weights_str)
+        else:
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return p._print(limit[0])
+                x, cond = limit
+                return r"%s \sim %s" % (p._print(x), p._print(cond))
+
         if not limits:
             tex += ' '  
         elif len(limits) == 1:
             limit = limits[0]
             if len(limit) == 1:
-                tex += r"_{%s} " % p._print(limit[0])
-            else:
-                tex += r"\limits_{%s \sim %s} " % tuple([p._print(i) for i in limit])
-        else:
-
-            def _format_ineq(limit):
-                if len(limit) == 1:
-                    return p._print(limit[0])
+                x, = limit
+                if weights:
+                    tex += r"_{%s \sim %s} " % (p._print(x), weights_str)
                 else:
-                    return r"%s \sim %s" % tuple([p._print(i) for i in limit])
-
+                    if {*self.expr.random_symbols} - {x}:
+                        tex += r"_{%s} " % p._print(x)
+            else:
+                tex += r"\limits_{%s} " % _format_ineq(limit)
+        else:
             if all(len(limit) == 1 for limit in limits):
-                tex += r"_{%s} " % str.join(', ', [p._print(l[0]) for l in limits])
+                vars = [limit[0] for limit in limits]
+                if {*self.expr.random_symbols} - {*vars}:
+                    tex += r"_{%s} " % str.join(', ', [p._print(v) for v in vars])
             else:
                 tex += r"\limits_{\begin{subarray}{c}%s\end{subarray}} " % str.join('\\\\', [_format_ineq(l) for l in limits])
 
-        if self.expr.is_Add or self.expr.is_Conditioned:
-            tex += r"\left(%s\right)" % p._print(self.expr)
-        else:
-            tex += p._print(self.expr)
-
+        tex += r"\left(%s\right)" % p._print(self.expr)
         return tex
 
+    def _sympystr(self, p):
+        if weights := self.weights:
+            weights_str = ','.join([p._print(w) for w in weights])
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return r"%s ~ %s" % (p._print(limit[0]), weights_str)
+                x, cond = limit
+                return r"%s ~ %s | %s" % (p._print(x), p._print(cond), weights_str)
+        else:
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return p._print(limit[0])
+                x, cond = limit
+                return r"%s ~ %s" % (p._print(x), p._print(cond))
 
-class Variance(Expr):
+        return 'E[%s](%s)' % (','.join([_format_ineq(limit) for limit in self.limits]), p._print(self.expr))
+
+    def _eval_is_random(self):
+        expr = self.expr
+        
+        variables_set = self.variables_set
+        if any(not any(V.index_contains(v) for V in variables_set) for v in expr.random_symbols): 
+            return True
+                
+        if expr.is_Conditioned:
+            for v in expr.rhs.yield_random_symbols():
+                if v.is_Surrogate:
+                    return True
+
+    def _if_new_has_variables(self, old, new):
+        return set()
+    
+    def simplify(self, deep=False):
+        [*limits] = self.limits
+        expr = self.expr
+        if expr.is_Conditioned:
+            expr = expr.lhs
+            
+        if std.deleteIndices(limits, lambda limit: not expr._has(limit[0])):
+            if weights := self.weights:
+                limits.append(weights)
+            self = self.func(self.expr, *limits)
+            
+        random_symbols = expr.random_symbols
+        
+        varMapping = defaultdict(list)
+        for x, *ab in self.limits:
+            if ab:
+                continue
+            
+            for v in random_symbols:
+                if x != v and x.index_contains(v):
+                    varMapping[x].append(v)
+                    
+        if varMapping:
+            for x, s in varMapping.items():
+                if len(s) == 1:
+                    s, = s
+                    i = self.variables.index(x)
+                    limits[i] = (s,)
+                
+            self = self.func(self.expr, *limits, *self.weights)
+
+        return ExprWithLimits.simplify(self, deep)
+
+    def yield_random_symbols_from_expr(self):
+        expr = self.expr
+        if expr.is_Conditioned:
+            lhs, rhs = expr.args
+            yield from lhs.yield_random_symbols()
+
+            for v in rhs.yield_random_symbols():
+                if v.is_Surrogate:
+                    yield v.arg
+        else:
+            yield from expr.yield_random_symbols()
+
+    def yield_random_symbols(self):
+        variables = self.variables
+        for v in self.yield_random_symbols_from_expr():
+            if not any(V.index_contains(v) for V in variables):
+                yield v
+
+        for x, *ab in self.limits:
+            for v in ab:
+                yield from v.yield_random_symbols()
+
+    def _has_indexed(self, pattern):
+        base = pattern.base
+        if base.is_Indexed:
+            base = base.base
+        
+        if not base.is_random:
+            for surrogate in self.finditer(Surrogate):
+                if surrogate.arg.var._has(pattern):
+                    return True
+
+        return ExprWithLimits._has_indexed(self, pattern)
+
+    def __iter__(self):
+        raise TypeError
+    
+    def __getitem__(self, indices, **kw_args):
+        return self.func(self.expr[indices], *self.limits, *self.weights)
+
+
+class Variance(ExprWithLimits):
     """
     Symbolic expression for the variance.
 
@@ -670,19 +1148,18 @@ class Variance(Expr):
 
     """
 
-    def __new__(cls, arg, condition=None, **kwargs):
-        arg = _sympify(arg)
-
-        if arg.is_Matrix:
-            from sympy.stats.symbolic_multivariate_probability import VarianceMatrix
-            return VarianceMatrix(arg, condition)
-        if condition is None:
-            obj = Expr.__new__(cls, arg)
-        else:
-            condition = _sympify(condition)
-            obj = Expr.__new__(cls, arg, condition)
-        obj._condition = condition
-        return obj
+    def __new__(cls, expr, *limits, **kwargs):
+        expr = _sympify(expr)
+        if not expr.is_random:
+            from sympy import ZeroMatrix
+            return ZeroMatrix(*expr.shape)
+        
+        if (given := kwargs.get('given')) is not None:
+            if given.is_symbol:
+                given = Equal(given, given.var)
+            expr = Conditioned(expr, given)
+        
+        return ExprWithLimits.__new__(cls, expr, *limits)
 
     def expand(self, **hints):
         arg = self.args[0]
@@ -733,8 +1210,130 @@ class Variance(Expr):
     def evaluate_integral(self):
         return self.rewrite(Integral).doit()
 
+    def _latex(self, p):
+        tex = r'\mathop{\mathbb{Var}}'
+        limits = self.limits
+        
+        def _format_ineq(limit):
+            if len(limit) == 1:
+                return p._print(limit[0])
+            x, cond = limit
+            return r"%s \sim %s" % (p._print(x), p._print(cond))
+        
+        if not limits:
+            tex += ' '  
+        elif len(limits) == 1:
+            limit = limits[0]
+            if len(limit) == 1:
+                x, = limit
+                if {*self.expr.random_symbols} - {x}:
+                    tex += r"_{%s} " % p._print(x)
+            else:
+                tex += r"\limits_{%s} " % _format_ineq(limit)
+        else:
+            if all(len(limit) == 1 for limit in limits):
+                vars = [limit[0] for limit in limits]
+                if {*self.expr.random_symbols} - {*vars}:
+                    tex += r"_{%s} " % str.join(', ', [p._print(v) for v in vars])
+            else:
+                tex += r"\limits_{\begin{subarray}{c}%s\end{subarray}} " % str.join('\\\\', [_format_ineq(l) for l in limits])
 
-class Covariance(Expr):
+        tex += r"\left(%s\right)" % p._print(self.expr)
+        return tex
+
+    def _sympystr(self, p):
+        expr, *limits = self.args
+        if limits:
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return p._print(limit[0])
+                x, cond = limit
+                return r"%s ~ %s" % (p._print(x), p._print(cond))
+            
+            return 'Var[%s](%s)' % (','.join([_format_ineq(limit) for limit in limits]), p._print(expr))
+        else:
+            return 'Var(%s)' % p._print(expr)
+
+    def _eval_is_random(self):
+        if {*self.expr.random_symbols} - {*self.variables}:
+            return True
+        
+        return False
+
+
+class KL(Expr):
+    """
+    Symbolic expression for the Kullback-Leibler divergence.
+    """
+
+    def __new__(cls, lhs, rhs, **kwargs):
+        assert lhs.is_Probability and rhs.is_Probability
+        return Expr.__new__(cls, lhs, rhs)
+
+    def _latex(self, p):
+        tex = r'\mathop{\mathbb{KL}}'
+        lhs, rhs = self.args
+        limits = []
+        
+        def _format_ineq(limit):
+            if len(limit) == 1:
+                return p._print(limit[0])
+            x, cond = limit
+            return r"%s \sim %s" % (p._print(x), p._print(cond))
+        
+        if not limits:
+            tex += ' '  
+        elif len(limits) == 1:
+            limit = limits[0]
+            if len(limit) == 1:
+                x, = limit
+                if {*self.expr.random_symbols} - {x}:
+                    tex += r"_{%s} " % p._print(x)
+            else:
+                tex += r"\limits_{%s} " % _format_ineq(limit)
+        else:
+            if all(len(limit) == 1 for limit in limits):
+                vars = [limit[0] for limit in limits]
+                if {*self.expr.random_symbols} - {*vars}:
+                    tex += r"_{%s} " % str.join(', ', [p._print(v) for v in vars])
+            else:
+                tex += r"\limits_{\begin{subarray}{c}%s\end{subarray}} " % str.join('\\\\', [_format_ineq(l) for l in limits])
+        #\bigg/
+        tex += r"\left(%s\middle/%s\right)" % (p._print(lhs), p._print(rhs))
+        return tex
+
+    @property
+    def shape(self):
+        return ()
+    
+    @property
+    def dtype(self):
+        from sympy import dtype
+        return dtype.real
+
+    def _eval_is_extended_nonnegative(self):
+        return True
+
+    def _sympystr(self, p):
+        lhs, rhs = self.args
+        limits = []
+
+        if limits:
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return p._print(limit[0])
+                x, cond = limit
+                return r"%s ~ %s" % (p._print(x), p._print(cond))
+            
+            return 'KL[%s](%s, %s)' % (','.join([_format_ineq(limit) for limit in limits]), p._print(lhs), p._print(rhs))
+        else:
+            return 'KL(%s, %s)' % (p._print(lhs), p._print(rhs))
+
+    def _eval_is_random(self):
+        return any(arg.is_random for arg in self.args)
+
+
+class Covariance(ExprWithLimits):
     """
     Symbolic expression for the covariance.
 
@@ -779,23 +1378,27 @@ class Covariance(Expr):
     a*b*Covariance(X, Y)
     """
 
-    def __new__(cls, arg1, arg2, condition=None, **kwargs):
+    def __new__(cls, *args, **kwargs):
+        if args[0].is_Tuple:
+            arg1, arg2 = args[0]
+            limits = args[1:]
+        else:
+            arg1, arg2, *limits = args
         arg1 = _sympify(arg1)
         arg2 = _sympify(arg2)
 
-        if arg1.is_Matrix or arg2.is_Matrix:
-            from sympy.stats.symbolic_multivariate_probability import CrossCovarianceMatrix
-            return CrossCovarianceMatrix(arg1, arg2, condition)
-
         if kwargs.pop('evaluate', global_parameters.evaluate):
             arg1, arg2 = sorted([arg1, arg2], key=default_sort_key)
+            
+        from sympy import Tuple
+        expr = Tuple(arg1, arg2)
+        if (given := kwargs.get('given')) is not None:
+            if given.is_symbol:
+                given = Equal(given, given.var)
+                
+            expr = Conditioned(expr, given)
 
-        if condition is None:
-            obj = Expr.__new__(cls, arg1, arg2)
-        else:
-            condition = _sympify(condition)
-            obj = Expr.__new__(cls, arg1, arg2, condition)
-        obj._condition = condition
+        obj = ExprWithLimits.__new__(cls, expr, *limits)
         return obj
 
     def expand(self, **hints):
@@ -869,6 +1472,55 @@ class Covariance(Expr):
     def evaluate_integral(self):
         return self.rewrite(Integral).doit()
 
+    def _latex(self, p):
+        (arg1, arg2), *limits = self.args
+        tex = r'\mathop{\mathbb{Cov}}'
+        def _format_ineq(limit):
+            if len(limit) == 1:
+                return p._print(limit[0])
+            x, cond = limit
+            return r"%s \sim %s" % (p._print(x), p._print(cond))
+        
+        if not limits:
+            tex += ' '  
+        elif len(limits) == 1:
+            limit = limits[0]
+            if len(limit) == 1:
+                tex += r"_{%s} " % p._print(limit[0])
+            else:
+                tex += r"\limits_{%s} " % _format_ineq(limit)
+        else:
+            if all(len(limit) == 1 for limit in limits):
+                tex += r"_{%s} " % str.join(', ', [p._print(l[0]) for l in limits])
+            else:
+                tex += r"\limits_{\begin{subarray}{c}%s\end{subarray}} " % str.join('\\\\', [_format_ineq(l) for l in limits])
+
+        tex += r"\left(%s,%s\right)" % (p._print(arg1), p._print(arg2))
+        return tex
+
+    def _sympystr(self, p):
+        expr, *limits = self.args
+        if limits:
+            def _format_ineq(limit):
+                if len(limit) == 1:
+                    return p._print(limit[0])
+                x, cond = limit
+                return r"%s ~ %s" % (p._print(x), p._print(cond))
+            
+            return 'Cov[%s]%s' % (','.join([_format_ineq(limit) for limit in limits]), p._print(expr))
+        else:
+            return 'Cov%s' % p._print(expr)
+
+    @property
+    def dtype(self):
+        return self.expr[0].dtype
+    
+    def _eval_is_random(self):
+        if {*self.expr.random_symbols} - {*self.variables}:
+            return True
+        
+        return False
+
 
 class Moment(Expr):
     """
